import matplotlib as mpl
from mpl_toolkits.mplot3d import Axes3D
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import time

# Einstellungen fuer die Grafik:
fig = plt.figure()
ax = fig.add_axes([0,0,1,0.9], projection='3d',label='axes3')	# 3D Grafik
ax_head = fig.add_axes([0,0.9,1,1],label='axes_head')			# Kopfgrafik, in der Zeit dargestellt wird
ax.set_aspect('equal')

class Himmelskoerper(object):
	
	Liste_der_Kraefte = []			# die Liste der Kraefte
	m = 1.0							# Masse des Himmelskoerpers
	v = np.array([0.0,0.0,0.0])		# Geschwindigkeitsvektor
	ort = np.array([0.0,0.0,0.0])	# Ortsvektor
	name = ""						# Name des Himmelskoerpers
	
	def __repr__(self):
		return self.name
		
	def __init__(self,name,m):
		self.name = name
		self.m = m

	# Ab hier beginnen die Methoden zur Ortsberechnung. Damit man nicht jede Methode einzeln aufrufen muss,
	# greift jede Methode auf die jeweils dafuer benoetigte Methode zu, sodass am Ende im Programm nur die letzte Methode benutzt werden muss
	
	def berechne_gravitationskraft(self,objekt1):
		"""
		Gibt die Gravitationskraft aus, die zwischen den Objekten "objekt1" und "objekt2" herrscht
		Das Ergebnis wird als Vektor ausgegeben, immer in Richtung von objekt1 bis objekt2
		"""
		verbindungsvektor = self.ort-objekt1.ort
		betrag = np.linalg.norm(verbindungsvektor)
		F_betrag = (welt.gravitationskonstante * objekt1.m * self.m)/betrag**2
		F = (F_betrag/betrag)*verbindungsvektor
		return F
		
	def berechne_kraefte_neu(self,welt):
		"""
		Aktualisiert die Werte in der Liste der Kraefte, indem alle Werte neu berechnet werden
		"""
		self.Liste_der_Kraefte = []
		for i in welt.Liste_der_Objekte:
			if i.name != self.name:
				k = i.berechne_gravitationskraft(self)
				self.Liste_der_Kraefte.append(k)
	
	def berechne_resultierende_Kraft(self,welt):
		"""
		Aus der Liste der Kraefte wird eine resultierende Kraft in Form eines Vektors berechnet
		"""
		self.berechne_kraefte_neu(welt)
		r = np.array([0.0,0.0,0.0])			#resultierende Kraft
		for i in self.Liste_der_Kraefte:
			r = r + i
		return r
		
	def berechne_beschleunigung(self,welt):
		"""
		Aus der resultierende Kraft wird mit der Masse die Beschleunigung berechnet
		"""
		r = self.berechne_resultierende_Kraft(welt)
		a = r/self.m
		return a
		
	def berechne_geschwindigkeitsvektor(self,zeitschritt,welt):
		"""
		Aus der Beschleunigung wird mit dem uebergebenen Zeitschritt der Geschwindigkeitsvektor berechnet
		"""
		a = self.berechne_beschleunigung(welt)
		v = a*zeitschritt
		return self.v + v					#alte und neue Geschwindigkeit addiert
		
	def berechne_ort(self,zeitschritt,welt):
		"""
		Nun wird wieder mithilfe des Zeitschrittes aus der Geschwindigkeit die zurueckgelegte Strecke berechnet und daraus folgt dann der neue Ort
		"""
		v = self.berechne_geschwindigkeitsvektor(zeitschritt,welt)
		self.v = v
		s = v*zeitschritt								#zurueckgelegte Strecke im Zeitintervall
		ort = self.ort + s
		return ort
		
	def leapfrog(self,zeitschritt,welt):
		"""
		Die Methode berechne_ort ist eigentlich schon ausreichend, allerdings gibt es Probleme mit der Genauigkeit.
		Hier wird das geloest, indem das Leapfrog Verfahren angewendet wird.
		"""
		ort_zwischen = self.ort + self.v * (zeitschritt/2.0)
		self.ort = ort_zwischen
		a_zwischen = self.berechne_beschleunigung(welt)
		self.v = self.v + a_zwischen * zeitschritt
		self.ort = ort_zwischen + self.v * (zeitschritt/2.0)
		return self.ort

		
#------------------------------------------------------------------------------------------------------------------------------------

class Welt(object):
	                       
	gravitationskonstante = 6.673 * 10**(-11)			#Gravitationskonstante
	Liste_der_Objekte = []

	


	def objekt_hinzufuegen(self,objekt,ort):
		"""
		Fuegt der Welt ein Objekt hinzu
		"""
		self.Liste_der_Objekte.append(objekt)
		objekt.ort = ort
		
	def objekt_als_satellit_hinzufuegen(self,objekt,satellit,abstand):
		"""
		Fuegt ein Objekt ("satellit" genannt) in die Welt ein, welches sich
		um ein anderes Objekt ("objekt" genannt) dreht. Es ist dabei in einem stabilen Orbit.
		Der Abstand ("abstand" genannt) ist dabei der Abstand zwischen den Mittelpunkten.
		Ansatz: F_g = F_r
		Nach aufloesung erhaelt man: v = sqrt(gravitationskonstante*masse_objekt/abstand)
		"""
		satellit.v = np.array([0.0,(self.gravitationskonstante*objekt.m/abstand)**0.5,0.0])
		ort = objekt.ort + np.array([abstand,0,0])
		self.objekt_hinzufuegen(satellit,ort)

#------------------------------------------------------------------------------------------------------------------------------------

sonne = Himmelskoerper("Sonne",1.989E30)								# die Sonne wird mit korrekter Masse hinzugefuegt
merkur = Himmelskoerper("Merkur",3.301E23)								# der Merkur wird mit korrekter Masse hinzugefuegt
venus = Himmelskoerper("Venus",4.869E24)								# die Venus wird mit korrekter Masse hinzugefuegt
erde = Himmelskoerper("Erde",5.972E24)									# die Erde wird mit korrekter Masse hinzugefuegt
mars = Himmelskoerper("Mars",6.419E23)									# der Mars wird mit korrekter Masse hinzugefuegt
welt = Welt()															# es wird eine Welt definiert, in die Objekte eingefuegt werden koennen

welt.objekt_hinzufuegen(sonne,np.array([0.0,0.0,0.0]))					# die Sonne wird in der Mitte eingefuegt
welt.objekt_als_satellit_hinzufuegen(sonne,merkur,58000000000.0)		# der Merkur wird mit korrektem Radius als Satellit um die Sonne kreisend eingefuegt
welt.objekt_als_satellit_hinzufuegen(sonne,venus,108000000000.0)		# die Venus wird mit korrektem Radius als Satellit um die Sonne kreisend eingefuegt
welt.objekt_als_satellit_hinzufuegen(sonne,erde,149600000000.0)			# die Erde wird mit korrektem Radius als Satellit um die Sonne kreisend eingefuegt
welt.objekt_als_satellit_hinzufuegen(sonne,mars,228000000000.0)			# der Mars wird mit korrektem Radius als Satellit um die Sonne kreisend eingefuegt
"""
Fuer die artist Animation werden leere Listen erzeugt, in die spaeter die einzelnen Positionen der Planeten eingefuegt werden.
"""
merkurbahn = []
venusbahn = []
erdbahn = []
marsbahn = []
Jahr = 31536000				        # 31.536.000 Sekunden entsprechen etwa einem Jahr
Zeit = Jahr/100						# die laenge eines Zeitschritts in Sekunden
"""
Hier werden die einzelnen Planetenpositionen pro Zeitschritt berechnet. Dies geschieht 100 Mal, und da ein Zeitschritt genau ein hundertstel Erdjahr ist, wird in der Simulation auch ein Erdjahr simuliert.
Die kann man gut daran erkennen, dass die anderen Planeten zwar springen und die Animation von vorne beginnen, die Erde aber eine fluessige Bewegung hat.
Es werden die Orte der Planeten durch "leapfrog" bestimmt, danach werden die aktuellen Ortskoordinaten der Planeten an die List fuer die Animation gehaengt
"""
for i in xrange(100):
	merkur.ort = merkur.leapfrog(Zeit,welt)
	venus.ort = venus.leapfrog(Zeit,welt)
	erde.ort = erde.leapfrog(Zeit,welt)
	mars.ort = mars.leapfrog(Zeit,welt)
	ort0 = ax.plot3D(np.array([merkur.ort[0]]),np.array([merkur.ort[1]]),np.array([merkur.ort[2]]),'b.')
	merkurbahn.append(ort0)
	ort0 = ax.plot3D(np.array([venus.ort[0]]),np.array([venus.ort[1]]),np.array([venus.ort[2]]),'r.')
	venusbahn.append(ort0)
	ort0 = ax.plot3D(np.array([erde.ort[0]]),np.array([erde.ort[1]]),np.array([erde.ort[2]]),'b.')
	erdbahn.append(ort0)
	ort0 = ax.plot3D(np.array([mars.ort[0]]),np.array([mars.ort[1]]),np.array([mars.ort[2]]),'r.')
	marsbahn.append(ort0)

#--------------Grafik----------------------------------------------------------------------------------------------------------------

ax.plot3D(np.array([sonne.ort[0]]),np.array([sonne.ort[1]]),np.array([sonne.ort[2]]),'yo')		# die Sonne wird als fester Punkt in der Mitte dargestellt
AnimationMerkur = animation.ArtistAnimation(fig, merkurbahn,100)								# Aus der Liste mit den Merkurpositionen wird nun eine Animation gemacht
AnimationVenus = animation.ArtistAnimation(fig, venusbahn,100)									# Aus der Liste mit den Venuspositionen wird nun eine Animation gemacht
AnimationErde = animation.ArtistAnimation(fig, erdbahn,100)										# Aus der Liste mit den Erdpositionen wird nun eine Animation gemacht
AnimationMars = animation.ArtistAnimation(fig, marsbahn,100)									# Aus der Liste mit den Marspositionen wird nun eine Animation gemacht
# Werden die Zeilen mit der Animation weggelassen werden die einzelnen Positionen gleichzeitig und dauerhaft angezeigt

#Timer ----------------------------------------------------------------------------------------

zeit0 = time.time()										# die aktuelle Zeit in Sekunden, von der die Differenz berechnet wird

def update_title(axes):									# eine Methode, die die verstrichene Zeit seit "zeit0" ausgibt
    timetext.set_text((time.time() - zeit0))
    fig.canvas.draw()

timetext = ax_head.text(0.2,0.04,'0.0')					# der erste Wert, 0.0 wird festgelegt

timer = fig.canvas.new_timer(interval=100)				# der Timer wird 10 Mal in einer Sekunde aktualisiert
timer.add_callback(update_title, ax)					# die Aktualisierung greift auf die Methode zu, die oben definiert wurde
timer.start()

#Timer Ende -----------------------------------------------------------------------------------

grenze = 250000000000									# damit die Grafik nicht verzerrt aussieht, werden hier die Grenzen gleich gross festgelegt
ax.set_xlim(-grenze,grenze)
ax.set_ylim(-grenze,grenze)
ax.set_zlim(-grenze,grenze)

plt.show()												# die Grafik wird angezeigt
